@@ -1,8 +1,9 @@
 from .gql_handler import *
 from datetime import datetime
+import asyncio
 
 class PyXploraApi:
-    def __init__(self, countryPhoneNumber: str, phoneNumber: str, password: str, userLang: str, timeZone: str, watchNo: int=0):
+    def __init__(self, countryPhoneNumber: str, phoneNumber: str, password: str, userLang: str, timeZone: str, watchNo: int=0) -> None:
         self.handler = GQLHandler(countryPhoneNumber, phoneNumber, password, userLang, timeZone)
         self.handler.login()
 
@@ -21,7 +22,7 @@ class PyXploraApi:
         self.school_silent_mode = []
 
 ##### Contact Info #####
-    def getContacts(self):
+    def getContacts(self) -> list:
         for contact in self.handler.getContacts(self.watch_user_id)['contacts']['contacts']:
             try:
                 xcoin = contact['contactUser']['xcoin']
@@ -41,29 +42,35 @@ class PyXploraApi:
         return self.contacts
 
 ##### User Info #####
-    def getUserID(self):
+    def getUserID(self) -> str:
         return self.myInfo['id']
-    def getUserName(self):
+    def getUserName(self) -> str:
         return self.myInfo['name']
-    def getUserIcon(self):
+    def getUserIcon(self) -> str:
         return self.myInfo['extra']['profileIcon']
-    def getUserXcoin(self):
+    def getUserXcoin(self) -> int:
         return self.myInfo['xcoin']
-    def getUserCurrentStep(self):
+    def getUserCurrentStep(self) -> int:
         return self.myInfo['currentStep']
-    def getUserTotalStep(self):
+    def getUserTotalStep(self) -> int:
         return self.myInfo['totalStep']
-    def getUserCreate(self):
+    def getUserCreate(self) -> str:
         return datetime.fromtimestamp(self.myInfo['create']).strftime('%Y-%m-%d %H:%M:%S')
-    def getUserUpdate(self):
+    def getUserUpdate(self) -> str:
         return datetime.fromtimestamp(self.myInfo['update']).strftime('%Y-%m-%d %H:%M:%S')
 
 ##### Watch Info #####
-    def getWatchCurrentStep(self):
+    def getWatchUserID(self) -> str:
+        return self.watch_user_id
+    def getWatchUserName(self) -> str:
+        return self.watch_user_name
+    def getWatchXcoin(self) -> int:
+        return self.myInfo['children'][self.watch_no]['ward']['xcoin']
+    def getWatchCurrentStep(self) -> int:
         return self.myInfo['children'][self.watch_no]['ward']['currentStep']
-    def getWatchTotalStep(self):
+    def getWatchTotalStep(self) -> int:
         return self.myInfo['children'][self.watch_no]['ward']['totalStep']
-    def getWatchAlarm(self):
+    def getWatchAlarm(self) -> list:
         for alarm in self.handler.getAlarms(self.watch_user_id)['alarms']:
             self.alarms.append({
                 'name': alarm['name'],
@@ -72,21 +79,27 @@ class PyXploraApi:
                 'status': alarm['status'],
             })
         return self.alarms
-    def getWatchUserID(self):
-        return self.watch_user_id
-    def getWatchUserName(self):
-        return self.watch_user_name
-    def getWatchXcoin(self):
-        return self.myInfo[0]['ward']['xcoin']
-    def getWatchBattery(self):
+    def getWatchBattery(self) -> int:
         return self.watch_last_location['battery']
-    def getWatchIsCharging(self):
+    async def getWatchBattery_a(self) -> int:
+        await self.askWatchLocate_async()
+        await asyncio.sleep(15)
+        if (int(datetime.timestamp(datetime.now())) - self.watch_last_location['tm']) < 30:
+            return self.watch_last_location['battery']
+        watch_location = (await self.askHelper())
+        await asyncio.sleep(1)
+        return watch_location['battery']
+        
+    def getWatchIsCharging(self) -> bool:
         return self.watch_last_location['isCharging']
-    def getWatchOnlineStatus(self):
-        return self.handler.getWatches(self.watch_user_id)['watches'][self.watch_no]['onlineStatus']
-    def getWatchUnReadChatMsgCount(self): # bug?
+    def getWatchOnlineStatus(self) -> WatchOnlineStatus:
+        try:
+            return self.handler.getWatches(self.watch_user_id)['watches'][self.watch_no]['onlineStatus']
+        except TypeError:
+            return WatchOnlineStatus.UNKNOWN.value
+    def getWatchUnReadChatMsgCount(self) -> int: # bug?
         return self.handler.unReadChatMsgCount(self.watch_user_id)['unReadChatMsgCount']
-    def getWatchChats(self): # bug?
+    def getWatchChats(self) -> list: # bug?
         for chat in self.handler.chats(self.watch_user_id)['chats']['list']:
             self.chats.append({
                 'msgId': chat['msgId'],
@@ -104,26 +117,41 @@ class PyXploraApi:
         return self.chats
 
 ##### Watch Location Info #####
-    def getWatchLastLocation(self):
+    def getWatchLastLocation(self) -> dict:
         return self.watch_last_location
-    def getWatchLocateType(self):
+    def getWatchLocateType(self) -> str:
         return self.watch_last_location['locateType']
-    def getWatchLocate(self):
+    async def getWatchLocate(self) -> dict:
+        await self.askWatchLocate_async()
+        await asyncio.sleep(15)
+        if (int(datetime.timestamp(datetime.now())) - self.watch_last_location['tm']) < 30:
+            return {
+                'tm': datetime.fromtimestamp(self.watch_last_location['tm']).strftime('%Y-%m-%d %H:%M:%S'),
+                'lat': self.watch_last_location['lat'],
+                'lng': self.watch_last_location['lng'],
+                'rad': self.watch_last_location['rad'],
+                'poi': self.watch_last_location['poi'],
+                'city': self.watch_last_location['city'],
+                'province': self.watch_last_location['province'],
+                'country': self.watch_last_location['country'],
+            }
+        watch_location = (await self.askHelper())
+        await asyncio.sleep(1)
         return {
-            'tm': datetime.fromtimestamp(self.watch_last_location['tm']).strftime('%Y-%m-%d %H:%M:%S'),
-            'lat': self.watch_last_location['lat'],
-            'lng': self.watch_last_location['lng'],
-            'rad': self.watch_last_location['rad'],
-            'poi': self.watch_last_location['poi'],
-            'city': self.watch_last_location['city'],
-            'province': self.watch_last_location['province'],
-            'country': self.watch_last_location['country'],
+            'tm': datetime.fromtimestamp(watch_location['tm']).strftime('%Y-%m-%d %H:%M:%S'),
+            'lat': watch_location['lat'],
+            'lng': watch_location['lng'],
+            'rad': watch_location['rad'],
+            'poi': watch_location['poi'],
+            'city': watch_location['city'],
+            'province': watch_location['province'],
+            'country': watch_location['country'],
         }
-    def getWatchIsInSafeZone(self):
+    def getWatchIsInSafeZone(self) -> bool:
         return self.watch_last_location['isInSafeZone']
-    def getWatchSafeZoneLabel(self):
+    def getWatchSafeZoneLabel(self) -> str:
         return self.watch_last_location['safeZoneLabel']
-    def getSafeZones(self):
+    def getSafeZones(self) -> list:
         for safeZone in self.handler.safeZones(self.watch_user_id)['safeZones']:
             self.safe_zones.append({
                 #safeZone,
@@ -135,13 +163,15 @@ class PyXploraApi:
                 'address': safeZone['address'],
             })
         return self.safe_zones
-    def trackWatchInterval(self):
+    def trackWatchInterval(self) -> int:
         return self.handler.trackWatch(self.watch_user_id)['trackWatch']
-    def askWatchLocate(self):
+    def askWatchLocate(self) -> bool:
         return self.handler.askWatchLocate(self.watch_user_id)['askWatchLocate']
+    async def askWatchLocate_async(self) -> bool:
+        return (await self.handler.askWatchLocate(self.watch_user_id))['askWatchLocate']
 
 ##### Feature #####
-    def schoolSilentMode(self):
+    def schoolSilentMode(self) -> list:
         for sientTime in self.handler.silentTimes(self.watch_user_id)['silentTimes']:
             self.school_silent_mode.append({
                 'id': sientTime['id'],
@@ -151,29 +181,34 @@ class PyXploraApi:
                 'status': sientTime['status'],
             })
         return self.school_silent_mode
-    def setEnableSilentTime(self, silentId):
-        return self.handler.setEnableSlientTime(silentId)['setEnableSilentTime']
-    def setDisableSilentTime(self, silentId):
-        return self.handler.setEnableSlientTime(silentId, NormalStatus.DISABLE)['setEnableSilentTime']
-    def setAllEnableSilentTime(self):
+    def setEnableSilentTime(self, silentId) -> bool:
+        return bool(self.handler.setEnableSlientTime(silentId)['setEnableSilentTime'])
+    def setDisableSilentTime(self, silentId) -> bool:
+        return bool(self.handler.setEnableSlientTime(silentId, NormalStatus.DISABLE)['setEnableSilentTime'])
+    def setAllEnableSilentTime(self) -> list:
         res = []
         for silentTime in self.schoolSilentMode():
-            res.append(self.setEnableSilentTime(silentTime['id']))
+            res.append(bool(self.setEnableSilentTime(silentTime['id'])))
         return res
     def setAllDisableSilentTime(self):
         res = []
         for silentTime in self.schoolSilentMode():
-            res.append(self.setDisableSilentTime(silentTime['id']))
+            res.append(bool(self.setDisableSilentTime(silentTime['id'])))
         return res
     def sendText(self, text): # sender is login User
         return self.handler.sendText(self.watch_user_id, text)
-    def shutdown(self):
+    def shutdown(self) -> bool:
         return self.handler.shutdown(self.watch_user_id)
-    def reboot(self):
+    def reboot(self) -> bool:
         return self.handler.reboot(self.watch_user_id)
 
 ##### - #####
-    def helperTime(self, time):
+    def helperTime(self, time) -> str:
         h = str(int(time) /60).split('.')
         h2 = str(int(h[1]) *60).zfill(2)[:2]
         return h[0].zfill(2) + ":" + str(h2).zfill(2)
+    async def askHelper(self) -> int:
+        try:
+            return (await self.handler.getWatchLastLocation_a(self.watch_user_id))['watchLastLocate']
+        except TypeError:
+            return 0
\ No newline at end of file
